Package: slam
Version: 0.1-55
Title: Sparse Lightweight Arrays and Matrices
Authors@R: c(person("Kurt", "Hornik", role = c("aut", "cre"),
                     email = "Kurt.Hornik@R-project.org",
		     comment = c(ORCID = "0000-0003-4198-9911")),
             person("David", "Meyer", role = "aut",
	             email = "David.Meyer@R-project.org",
		     comment = c(ORCID = "0000-0002-5196-3048")),
             person("Christian", "Buchta", role = "aut"))
Description: Data structures and algorithms for sparse arrays and matrices,
  based on index arrays and simple triplet representations, respectively.
Depends: R (>= 3.4.0)
Imports: stats
Enhances: Matrix, SparseM, spam
License: GPL-2
NeedsCompilation: yes
Packaged: 2024-11-12 16:45:23 UTC; hornik
Author: Kurt Hornik [aut, cre] (<https://orcid.org/0000-0003-4198-9911>),
  David Meyer [aut] (<https://orcid.org/0000-0002-5196-3048>),
  Christian Buchta [aut]
Maintainer: Kurt Hornik <Kurt.Hornik@R-project.org>
Repository: CRAN
Date/Publication: 2024-11-13 07:13:31 UTC
Built: R 4.4.0; amd64-portbld-midnightbsd4; 2026-07-18 06:33:51 UTC; unix
