# slam

[![Build Status](https://travis-ci.org/miguelgrinberg/slam.png?branch=master)](https://travis-ci.org/miguelgrinberg/slam)

Slam is a simple command-line utility that makes it very easy to do serverless
deployments of Python web applications to  [AWS](https://aws.amazon.com), using
the [Lambda](https://aws.amazon.com/lambda) and
[API Gateway](https://aws.amazon.com/api-gateway) services. In particular, Slam
supports the transparent deployment of [WSGI](https://www.python.org/dev/peps/pep-3333/)
compliant applications.

If you have 10 minutes to spare, you can watch a (now slightly outdated)
screencast in which the main features of slam are demonstrated:

[![Slam Demonstration](https://img.youtube.com/vi/9eoL6oGiodw/0.jpg)](https://www.youtube.com/watch?v=9eoL6oGiodw)

Or if you prefer something more formal, have a look at the 
[documentation](http://slam-python.readthedocs.io/en/latest/), which also
includes a written tutorial.
